<?php

namespace Drupal\entity_router\Plugin\EntityResponseHandler;

use Drupal\Core\Entity\EntityInterface;
use Drupal\entity_router\EntityResponseHandlerInterface;
use Drupal\jsonapi\Controller\RequestHandler;
use Drupal\jsonapi\ResourceType\ResourceTypeRepositoryInterface;
use Drupal\jsonapi\Routing\Routes;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * {@inheritdoc}
 *
 * @EntityResponseHandler(
 *   id = "api_json",
 *   dependencies = {"jsonapi"},
 * )
 */
class JsonApiEntityResponseHandler implements EntityResponseHandlerInterface {

  /**
   * A value of the "jsonapi.base_path" container parameter.
   *
   * @var string
   */
  protected $pathPrefix;
  /**
   * An instance of the "jsonapi.request_handler" service.
   *
   * @var \Drupal\jsonapi\Controller\RequestHandler
   */
  protected $requestHandler;
  /**
   * An instance of the "jsonapi.resource_type.repository" service.
   *
   * @var \Drupal\jsonapi\ResourceType\ResourceTypeRepositoryInterface
   */
  protected $resourceTypeRepository;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    string $path_prefix,
    RequestHandler $request_handler,
    ResourceTypeRepositoryInterface $resource_type_repository
  ) {
    $this->pathPrefix = trim($path_prefix, '/');
    $this->requestHandler = $request_handler;
    $this->resourceTypeRepository = $resource_type_repository;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new static(
      $container->getParameter('jsonapi.base_path'),
      $container->get('jsonapi.request_handler'),
      $container->get('jsonapi.resource_type.repository')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getResponse(Request $request, ?EntityInterface $entity): Response {
    if ($entity === NULL) {
      return new JsonResponse($entity);
    }

    $entity_type = $entity->getEntityTypeId();
    $resource_type = $this->resourceTypeRepository->get($entity_type, $entity->bundle());
    $resource_path = sprintf('%s/%s/%s', $this->pathPrefix, trim($resource_type->getPath(), '/'), $entity->uuid());

    // These properties are mandatory for the JSON API request handler.
    $request->attributes->set($entity_type, $entity);
    $request->attributes->set(Routes::RESOURCE_TYPE_KEY, $resource_type);
    // Mark the request in order to allow JSONAPI to recognize requests.
    /* @see \Drupal\jsonapi\Routing\Routes::getResourceTypeNameFromParameters() */
    $request->attributes->set(Routes::JSON_API_ROUTE_FLAG_KEY, TRUE);

    /* @var \Drupal\jsonapi\ResourceResponse $response */
    $response = $this->requestHandler->handle($request, $resource_type);
    // The path must be added as caching dependency because otherwise
    // we always will receive the same entity after first success.
    $response->addCacheableDependency($resource_path);

    return $response;
  }

}
